# Sakuin.pl
# Movable Type plugin for create sakuin and keyword links.
# by ideaman's Kunihiko Miyanaga

package MT::Plugin::Sakuin;
use vars qw( $VERSION );
$VERSION = "0.8";

my $indexes = <<INDEX;
あ行|^\xe3\x81[\x81-\x8a]|^\xe3\x82[\xa1-\xaa]
か行|^\xe3\x81[\x8b-\x94]|^\xe3\x82[\xab-\xb4]
さ行|^\xe3\x81[\x95-\x9e]|^\xe3\x82[\xb5-\xbe]
た行|^\xe3\x81[\x9f-\xa9]|^\xe3\x82\xbf|^\xe3\x83[\x80-\x89]
な行|^\xe3\x81[\xaa-\xae]|^\xe3\x83[\x8a-\x8e]
は行|^\xe3\x81[\xaf-\xbd]|^\xe3\x83[\x8f-\x9d]
ま行|^\xe3\x81[\xbe-\xbf]|^\xe8\x82[\x80-\x82]|^\xe3\x83[\x9e-\xa2]
や行|^\xe3\x82[\x83-\x88]|^\xe3\x83[\xa3-\xa8]
ら行|^\xe3\x82[\x89-\x8d]|^\xe3\x83[\xa9-\xad]
わ～ん|^\xe3\x82[\x8e-\x93]|^\xe3\x83[\xae-\xb6]
英数字|^[a-zA-Z0-9]
その他
INDEX

use MT;
use MT::Entry;
use MT::Template::Context;

use strict;
use lib 'plugins/Sakuin/lib';
require Sakuin;
my %sakuins = ();
my $sakuin;

unless ($@) {
	my $plugin = {
		name => "MTSakuin $VERSION",
		description => "Tags to create sakuin and keyword links.",
		doc_link => "http://www.ideamans.com/tool/sakuin.php" };
	MT->add_plugin(new MT::Plugin($plugin));
}

# Add tags of plugin.
# Container tags.
MT::Template::Context->add_container_tag('Sakuin' => sub{&_sakuin;});
MT::Template::Context->add_container_tag('SakuinKeywordLink' => sub{&_sakuin_keyword_link;});
MT::Template::Context->add_container_tag('SakuinGroups' => sub{&_sakuin_groups;});
MT::Template::Context->add_container_tag('SakuinKeywords' => sub{&_sakuin_keywords;});
MT::Template::Context->add_container_tag('SakuinKeywordEntries' => sub{&_sakuin_keyword_entries;});

MT::Template::Context->add_conditional_tag('SakuinIfPhoneticIsDifferent' => sub{&_sakuin_if_phonetic_is_different;});
MT::Template::Context->add_conditional_tag('SakuinGroupsHeader' => sub{&_sakuin_groups_header;});
MT::Template::Context->add_conditional_tag('SakuinGroupsFooter' => sub{&_sakuin_groups_footer;});
MT::Template::Context->add_conditional_tag('SakuinKeywordsHeader' => sub{&_sakuin_keywords_header;});
MT::Template::Context->add_conditional_tag('SakuinKeywordsFooter' => sub{&_sakuin_keywords_footer;});

MT::Template::Context->add_tag('SakuinGroup' => sub{&_stash('_sakuin_group', @_);});
MT::Template::Context->add_tag('SakuinGroupCount' => sub{&_stash('_sakuin_group_count', @_);});
MT::Template::Context->add_tag('SakuinGroupIndex' => sub{&_stash('_sakuin_group_idnex', @_);});
MT::Template::Context->add_tag('SakuinKeyword' => sub{&_stash('_sakuin_keyword', @_);});
MT::Template::Context->add_tag('SakuinKeywordPhonetic' => sub{&_stash('_sakuin_keyword_phonetic', @_);});
MT::Template::Context->add_tag('SakuinKeywordGroup' => sub{&_stash('_sakuin_keyword_group', @_);});
MT::Template::Context->add_tag('SakuinKeywordCount' => sub{&_stash('_sakuin_keyword_count', @_);});
MT::Template::Context->add_tag('SakuinKeywordIndex' => sub{&_stash('_sakuin_keyword_index', @_);});

# Build keyword
sub build_sakuin {
	my ($ctx, $blog_ids) = @_;
	
	# Default blog id is current blog id.
	$blog_ids = $ctx->stash('blog_id') unless $blog_ids;
	
	# Query sakuins hash by blog_ids.
	if( $sakuins{$blog_ids} ) {
		$sakuin = $sakuins{$blog_ids};
	} else {
		$sakuin = Sakuin->new;
		$sakuin->set_indexes($indexes);
		
		# Split blog ids.
		my @ids = split('\s*,\s*', $blog_ids);
	
		# Add entry for all blogs.
		foreach my $id (@ids) {
			my $entry_iter = MT::Entry->load_iter({blog_id => $id, status => MT::Entry::RELEASE() });
			while ( my $e = $entry_iter->() ) {
				next unless defined $e->keywords;
				$sakuin->add_entry($e->keywords, $e->id, $e->title, $e->permalink);
			}
		}
		
		# Build dictionary.
		$sakuin->build_dictionary;
		
		# Store sakuin object.
		$sakuins{$blog_ids} = $sakuin;
	}
}

# Handler of value tag.
sub _stash {
	my ($label, $ctx) = @_;
	defined(my $val = $ctx->stash($label)) || return '';
	$val;
}

# Handler of MTSakuinIfPhoneticIsDefferent tag.
sub _sakuin_if_phonetic_is_different {
	my ($ctx) = @_;

	# This tag must be written inside MTSakuin.
	return $ctx->error("MTSakuinIfPhoneticIsDefferent must be called in MTSakuinKeywords tag.") unless defined($ctx->stash('_sakuin_keyword_count')) || $sakuin;

	$ctx->stash('_sakuin_keyword') ne $ctx->stash('_sakuin_phonetic');
}

# Handler of MTSakuin tag.
sub _sakuin {
	my ($ctx, $args, $cond) = @_;
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	# Arguments: blog_ids
	my $blog_ids = $args->{'blog_id'};

	# Build sakuin object and dictionary.
	build_sakuin($ctx, $blog_ids);
	
	# Build content inside tag.
	defined($_ = $builder->build($ctx, $tokens)) || return $ctx->error($ctx->errstr);
	
	$_;
}

# Handler of MTSakuinLink tag.
sub _sakuin_keyword_link {
	my ($ctx, $args, $cond) = @_;
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	# This tag must be written inside MTSakuin.
	return $ctx->error("MTSakuinKeywordLink must be called in MTSakuin tag.") unless $sakuin;
	
	# Build content inside tag.
	defined($_ = $builder->build($ctx, $tokens)) || return $ctx->error($ctx->errstr);
	
	# Replace keyword to link.
	$sakuin->replace_keywords($_);
}

# Handler of MTSakuinGroups tag.
sub _sakuin_groups {
	my ($ctx, $args, $cond) = @_;
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	# This tag must be written inside MTSakuin.
	return $ctx->error("MTSakuinGroups must be called in MTSakuin tag.") unless $sakuin;
	
	# Arguments: groups
	my $groups = $args->{'groups'};
	
	# Select index array by groups.
	if( lc($groups) eq 'used' ) {
		$groups = $sakuin->{used_indexes};
	} elsif ( lc($groups) eq 'related' ) {
		$groups = $sakuin->{related_indexes};
	} else {
		$groups = $sakuin->{base_indexes};
	}
	
	return $ctx->error("Fail to get goups hash") unless $groups;
	
	# Loop indexes as groups.
	my $out = '';
	my $i = 0;
	
	foreach my $index (@$groups) {
		# Build and concat the content.
		local $ctx->{__stash}{_sakuin_group} = $index;
		local $ctx->{__stash}{_sakuin_group_count} = scalar(@$groups);
		local $ctx->{__stash}{_sakuin_group_index} = $i++;

		defined($_ = $builder->build($ctx, $tokens)) || return $ctx->error($ctx->errstr);
		$out .= $_;
	}
	
	$out;
}

# Handler of MTSakuinGroupsHeader.
sub _sakuin_groups_header {
	my $ctx = shift;

	# This tag must be written inside MTSakuinGroups.
	return $ctx->error("MTSakuinGroupsHeader must be called in MTSakuinGroups tag.") unless defined($ctx->stash('_sakuin_gorup_count')) || $sakuin;

	$ctx->stash('_sakuin_group_index') == 0;
}

# Handler of MTSakuinGroupsFooter.
sub _sakuin_groups_footer {
	my $ctx = shift;

	# This tag must be written inside MTSakuinGroups.
	return $ctx->error("MTSakuinGroupsFooter must be called in MTSakuinGroups tag.") unless defined($ctx->stash('_sakuin_group_count')) || $sakuin;

	$ctx->stash('_sakuin_group_index') == $ctx->stash('_sakuin_group_count') - 1;
}

# Handler of MTSakuinKeywords tag.
sub _sakuin_keywords {
	my ($ctx, $args, $cond) = @_;
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	# This tag must be written inside MTSakuin.
	return $ctx->error("MTSakuinKeywords must be called in MTSakuin tag.") unless $sakuin;
	
	# Arguments: keywords
	my $keywords = $args->{'keywords'};
	
	# Query keywords in current group.
	my $group = $ctx->stash('_sakuin_group');
	
	if( lc($keywords) eq 'related' ) {
		$keywords = $sakuin->query_related_keywords($group);
	} else {
		$keywords = $sakuin->query_used_keywords($group);
	}
	
	return $ctx->error("Fail to get keyword array.") unless $keywords;

	# Loop keywords.
	my $out = '';
	my $i = 0;
	
	foreach my $keyword (@$keywords) {
		local $ctx->{__stash}{_sakuin_keyword} = $keyword;
		local $ctx->{__stash}{_sakuin_keyword_phonetic} = $sakuin->get_keyword_phonetic($keyword);
		local $ctx->{__stash}{_sakuin_keyword_group} = $sakuin->get_keyword_index($keyword);
		local $ctx->{__stash}{_sakuin_keyword_count} = scalar(@$keywords);
		local $ctx->{__stash}{_sakuin_keyword_index} = $i++;

		# Build and concat contents.
		defined($_ = $builder->build($ctx, $tokens)) || return $ctx->error($ctx->errstr);
		$out .= $_;
	}
	
	$out;
}

# Handler of MTSakuinKeywordsHeader.
sub _sakuin_keywords_header {
	my $ctx = shift;

	# This tag must be written inside MTSakuinKeywords.
	return $ctx->error("MTSakuinKeywordsHeader must be called in MTSakuinKeywords tag.") unless defined($ctx->stash('_sakuin_keyword_count')) || $sakuin;

	$ctx->stash('_sakuin_keyword_index') == 0;
}

# Handler of MTSakuinKeywordsFooter.
sub _sakuin_keywords_footer {
	my $ctx = shift;

	# This tag must be written inside MTSakuinKeywords.
	return $ctx->error("MTSakuinKeywordsFooter must be called in MTSakuinKeywords tag.") unless defined($ctx->stash('_sakuin_keyword_count')) || $sakuin;

	$ctx->stash('_sakuin_keyword_index') == $ctx->stash('_sakuin_keyword_count') - 1;
}

# Handler of MTSakuinKeywordEntries.
sub _sakuin_keyword_entries {
	my ($ctx, $args, $cond) = @_;
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	# This tag must be written inside MTSakuin.
	return $ctx->error("MTSakuinKeywordEntries must be called in MTSakuinKeywords tag.") unless defined($ctx->stash('_sakuin_keyword'));
	
	# Query entries related keyword.
	my $keyword_entries = $sakuin->get_keyword_entries($ctx->stash('_sakuin_keyword'));
	
	# Build MT entries array.
	my @entries = ();
	foreach my $entry (@$keyword_entries) {
		my $e = MT::Entry->load($entry->{'id'});
		push(@entries, $e) if $e;
	}
	
	# Set entries array and transfer to MTEntries handler.
	local $ctx->{__stash}{entries} = \@entries;
	MT::Template::Context::_hdlr_entries(@_);
}

1;